#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>

#include "parith-15.h"

// Port fr PWM
#define PORT_PWM PORTB  // OC0A   
#define DDR_PWM  DDRB   // OC0A
#define PIN_PWM  0      // OC0A

#define IRQS_PER_SECOND  1000
#define PRESCALE 32

// Wird jede Millisekunde um 1 verringert und bleibt bei 0 stehen
volatile uint8_t count_ms;

static uint16_t seed __attribute__ ((section (".noinit")));
void __init_seed(void) __attribute__ ((naked, section (".init8")));

//static void ioinit(void); 
void timers_init(void);

/*****************************************************************/
// Timer0: PWM  @ F_CPU/256, d.h. 31.25 kHz
// Timer1: IRQs @ 1kHz, d.h. alle 1ms
void timers_init (void)
{
   // Timer1 in CTC-Mode und prescaled 32
//	TCCR1 = (1 << CTC1) | (1 << CS12) | (1 << CS11) | (0 << CS10); 
 
	// TIMER1 gibt 1ms-Takt vor
	// Precale 32 -> 8.000.000 / (16 * 256) = 976  -> ca. 0.5ms
	TCCR1 = (0 << CTC1) | (1 << CS12) | (0 << CS11) | (1 << CS10); 
	// Timer1overflowenable
	TIMSK = (1 << TOIE1);
	
   // OutputCompare-Interrupt B fr Timer1
//   TIMSK = (1 << OCIE1B);

   
   // outputCompare fr gewnschte Timer1-Frequenz
//   OCR1C = (uint8_t) ((uint32_t) F_CPU / IRQS_PER_SECOND / PRESCALE -1);
//   OCR1B = 0;
    

   // TIMER0 liefert das PWM-Signal
	// COM0A1 -> clear OC0A at compare match, set OC0A when downcounting
   // WGM00  -> phase correct, clear at top
	// Frequenz 8.000.000 / 510 * 1 = 15 KHz
   TCCR0A = (1 << WGM00) | (1 << COM0A1);
   TCCR0B = (0 << CS01) | (1 << CS00); // Prescale = 1


}

/*****************************************************************/
// ISR wird alle 1 ms ausgefhrt
// Verringert `count_ms' um 1 falls noch nicht 0
//SIGNAL (SIG_OUTPUT_COMPARE1B)
//ISR(TIMER1_COMPA_vect)
ISR(TIMER1_OVF_vect)
{

if (count_ms) count_ms--;

}    

/*****************************************************************/
// Initialize hardware
/*
void ioinit (void) 
{
    // high fuse = 0xcc (BOD = 4.3V, WDT=on)
 //   wdt_enable (WDTO_15MS);
    
    // Alle (nicht verwendeten) Ports sind IN + PullUp
//    DDRB  = 0;
//    PORTB = 0x3f;
   
    // Enable Timer1 OutputCompare-IRQ
    // Set    Timer0 PWM
    timers_init();

     // OCRA = OUT + LOW
    DDR_PWM  |=  (1 << PIN_PWM);    
//    PORT_PWM &= ~(1 << BIT_PWM);    
} */

/*****************************************************************/
int main (void)
	{
    // Initialisiere Hardware
   timers_init();
	DDR_PWM  |=  (1 << PIN_PWM); 
    
	CLKPR = (1<<CLKPCE);	// Clock-division-faktor auf 1 setzen (default=8 -> 1MHz))
	CLKPR = 0;

	register uint8_t i = 0;
	register uint8_t j;
   
	// Seed fr prandom() setzen
   // seed wurde vor main() durch __init_seed() bestimmt
   set_seed (seed);
    
   // IRQs sind willkommen
   sei();    
    
   uint8_t ocr = prandom();;
   uint8_t o   = 0;
    
   // main loop
   while (1)
		{
//    wdt_reset();

      // PWM wandert zum Sollwert ocr
      if (o < ocr) o++;
      if (o > ocr) o--;
      OCR0A = o;						// compare setzen
/*        
      if (count_ms)
         continue;

      // Zhler abgelaufen:
      // Zhler neu aufziehen
      count_ms = 100;
        
      // Neuer "zuflliger" PWM-Sollwert    
      ocr = prandom();
*/
      if (!count_ms)								// wenn counter_ms = 0
			{
         // Zhler neu aufziehen auf 50ms
         count_ms = 220;
 			i++;        
			if (i < 30) ocr = prandom();		// ruhigere Periode 30 Einheiten


			if (i == 30) ocr = 255;				// jetzt wird es turbulent
			if (i == 31) ocr = 0;				// 11 Einheiten
			if (i == 32) ocr = 255;
			if (i == 33) ocr = 0;
			if (i == 34) ocr = 255;
			if (i == 35) ocr = 0;
			if (i == 36) ocr = 255;
			if (i == 37) ocr = 0;
			if (i == 38) ocr = 255;
			if (i == 40) ocr = 0;
			if (i == 41) ocr = 255;

			if ((i > 42) && (i < 101))
				{
				ocr = prandom();					// wieder ruhiger 60 Einheiten
				}

			if (i == 101) ocr = 255;			// jetzt wird es turbulent
			if (i == 102) ocr = 0;				// 16 Einhiten
			if (i == 103) ocr = 255;
			if (i == 104) ocr = 0;
			if (i == 105) ocr = 255;
			if (i == 106) ocr = 0;
			if (i == 107) ocr = 255;
			if (i == 108) ocr = 0;
			if (i == 109) ocr = 255;
			if (i == 110) ocr = 0;
			if (i == 111) ocr = 255;
			if (i == 112) ocr = 0;
			if (i == 113) ocr = 255;
			if (i == 114) ocr = 0;			
			if (i == 115) ocr = 255;
			if (i == 116) ocr = 0;	
			if (i == 117) ocr = 255;		
			
			if (i > 117) i = 0;					// nchste Runde nach 117 * 1.2 Sekunden
			}
			
		j = 50;
		while (j--)
			{
			_delay_us(10);						// etwas die Hektik rausnehmen
			}
		}
	}

// Bestimme seed aus zuflligem RAM-Inhalt
// !!! FUNKTION NICHT AUFRUFEN       !!!
// !!! FUNKTION WIRD AUTOMATISCH     !!!
// !!! IN .init8 vor main AUSGEFHRT !!!
void __init_seed (void)
{
	uint16_t s = 0;
	uint16_t *p = (uint16_t*) (RAMEND+1);
	extern uint16_t __noinit_start;

	while (p >= &__noinit_start + 1)
		s ^= * (--p);
        
	seed = s;
}

/*
   while (1)
   {
       wdt_reset();

       // Zhler abgelaufen:
       if (!count_ms)
       {
           // Zhler neu aufziehen auf 100ms
           count_ms = 100;
           // Neuer, zuflliger PWM-Wert
           OCR0A = prandom();
       }
   }

*/
