/// -- [ lagerfeuer.c - zur Kerze mutiert ] -----------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>

#include "parith-15.h"

// Port fr PWM
#define PORT_PWM PORTB  // OC0A   
#define DDR_PWM  DDRB   // OC0A
#define PIN_PWM  0      // OC0A

#define IRQS_PER_SECOND  1000
#define PRESCALE 32

#define PWMSPEED1		120	// * 10us legt die Zeitabstnde zwischen 	
								// der kontinuierlichen nderungen des OC0A-Wertes
								// Geschwindigkeit der nderung
								// je kleiner, destro ruckhafter die nderung
#define PWMSPEED2		80
								
#define SPEED1			500 // * 500 us die Zeit bis zur Neufestsetzung eines
								//  Zielwertes
								// je kleiner, desto hektischer
#define SPEED2			200

// 200 * 500us / 50 * 10us = 100.000 / 500 = 20 nderungen								
// Wird jede 0.5 Millisekunde um 1 verringert und bleibt bei 0 stehen
volatile uint16_t count_ms;

static uint16_t seed __attribute__ ((section (".noinit")));
void __init_seed(void) __attribute__ ((naked, section (".init8")));

//static void ioinit(void); 
void timers_init(void);

/*****************************************************************/
// Timer0: PWM  @ F_CPU/256, d.h. 31.25 kHz
// Timer1: IRQs @ 1kHz, d.h. alle 1ms
void timers_init (void)
{
   // Timer1 in CTC-Mode und prescaled 32
//	TCCR1 = (1 << CTC1) | (1 << CS12) | (1 << CS11) | (0 << CS10); 
 
	// TIMER1 gibt 1ms-Takt vor
	// Precale 32 -> 8.000.000 / (16 * 256) = 976  -> ca. 0.5ms
	TCCR1 = (0 << CTC1) | (1 << CS12) | (0 << CS11) | (1 << CS10); 
	// Timer1overflowenable
	TIMSK = (1 << TOIE1);
	
   // OutputCompare-Interrupt B fr Timer1
//   TIMSK = (1 << OCIE1B);

   
   // outputCompare fr gewnschte Timer1-Frequenz
//   OCR1C = (uint8_t) ((uint32_t) F_CPU / IRQS_PER_SECOND / PRESCALE -1);
//   OCR1B = 0;
    

   // TIMER0 liefert das PWM-Signal
	// COM0A1 -> clear OC0A at compare match, set OC0A when downcounting
   // WGM00  -> phase correct, clear at top
	// Frequenz 8.000.000 / 510 * 1 = 15 KHz
   TCCR0A = (1 << WGM00) | (1 << COM0A1);
   TCCR0B = (0 << CS01) | (1 << CS00); // Prescale = 1


}

/*****************************************************************/
// ISR wird alle 0.5 ms ausgefhrt
// Verringert `count_ms' um 1 falls noch nicht 0
//SIGNAL (SIG_OUTPUT_COMPARE1B)
//ISR(TIMER1_COMPA_vect)
ISR(TIMER1_OVF_vect)
{

if (count_ms) count_ms--;

}    

/*****************************************************************/
// Initialize hardware
/*
void ioinit (void) 
{
    // high fuse = 0xcc (BOD = 4.3V, WDT=on)
 //   wdt_enable (WDTO_15MS);
    
    // Alle (nicht verwendeten) Ports sind IN + PullUp
//    DDRB  = 0;
//    PORTB = 0x3f;
   
    // Enable Timer1 OutputCompare-IRQ
    // Set    Timer0 PWM
    timers_init();

     // OCRA = OUT + LOW
    DDR_PWM  |=  (1 << PIN_PWM);    
//    PORT_PWM &= ~(1 << BIT_PWM);    
} */

/*****************************************************************/
int main (void)
{
// Initialisiere Hardware
timers_init();
DDR_PWM  |=  (1 << PIN_PWM); 
    
CLKPR = (1<<CLKPCE);	// Clock-division-faktor auf 1 setzen (default=8 -> 1MHz))
CLKPR = 0;

register uint8_t i = 0;
register uint8_t j;
register uint8_t pwmend;
   
// Seed fr prandom() setzen
// seed wurde vor main() durch __init_seed() bestimmt
set_seed (seed);
    

sei(); 											// IRQs sind willkommen   
    
uint8_t ocr = prandom();;
uint8_t o   = 0;

uint8_t time1[] = { 40, 91, 39, 37, 99, 60, 46, 37};		// ruhige Zeiten
uint8_t time2[] = { 13, 21, 23, 33, 16, 19, 29, 21, 17};	// Flackerzeiten
uint8_t limit1 = 0;													// bis hierhin ruhige Zeiten
uint8_t limit2 = 0;													// von limit1 bis hier Flackerzeiten

 
// main loop
while (1)
	{
// wdt_reset();


   if (o < ocr) o++;							 	// PWM wandert zum Sollwert ocr
   if (o > ocr) o--;
   OCR0A = o;										// compare setzen

   if (!count_ms)									// wenn counter_ms = 0
		{
         
      if (i == 0) 								// bei i == 0 neue Startwert auswhlen
			{
			ocr = prandom();
			limit1 = time1[prandom() & 0x07];// einen zuflligen Wert aus time1[] auswhlen
			limit2 = time2[prandom() & 0x07];// einen zuflligen Wert aus time2[] auswhlen
			limit2 += limit1;
			
			// auch noch etwas Zufall in die Kombination der Array bringen
			// time2[] nach rechts schieben
			time2[0] = time2[8];
			for (j = 0; j < 7; j++) time2[j+1] = time2[j];
			}
	
	// in der ruhigen Phase wird mit geringer Geschwindigkeit der Helligkeitsnderung (PWMSPEED)
	// und relativen langen Phasen (SPEED) bis zum setzen neuer Zielwerte gearbeitet.
		
		if (i < limit1) count_ms = SPEED1; else count_ms = SPEED2;
		i++;        
		if (i <= limit1) //ocr = prandom();		// ruhigere Periode per Zufall bestimmen
			{
			if (i & 0x01) ocr += 83; else ocr -= 57;  // oder durch Zhlen
			}
	
	
	// in der Flackerphase wird zwischen den Extremwerten gependelt
	// weiterhin wird Geschwindigkeit der Helligkeitsvernderung erhht 
		if (i > limit1 && i <= limit2)
			{
			if (i & 0x01) ocr = 255; else (ocr = 0);// jetzt wird es turbulent
			}
		if (i > limit2) i = 0;					// hier wird neu begonnen
		}	
		

	// hier wird eine Pause eingelegt, 
	// die die Geschwindigkeit der Helligkeitsvernderung regelt.
	if (i < (limit1 + 2)) pwmend = PWMSPEED1; else pwmend = PWMSPEED2;
	j = 0;
		
	while (j < pwmend)						// unterschiedliche Zeiten anwenden
		{
		j++;
		_delay_us(10);							// etwas Ruhe in die Flamme bringen
		}
	}
}

// Bestimme seed aus zuflligem RAM-Inhalt
// !!! FUNKTION NICHT AUFRUFEN       !!!
// !!! FUNKTION WIRD AUTOMATISCH     !!!
// !!! IN .init8 vor main AUSGEFHRT !!!
void __init_seed (void)
{
	uint16_t s = 0;
	uint16_t *p = (uint16_t*) (RAMEND+1);
	extern uint16_t __noinit_start;

	while (p >= &__noinit_start + 1)
		s ^= * (--p);
        
	seed = s;
}

